..
  Copyright (c) Ansible Project
  GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
  SPDX-License-Identifier: GPL-3.0-or-later

{% for i in examples %}
{{ i.label }}

.. code-block:: {{ i.lang }}

  {{ lookup('file', i.file)|indent(2) }}

{% endfor %}
