{#
Copyright (c) Ansible Project
GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
SPDX-License-Identifier: GPL-3.0-or-later
#}
{
  "name": "ansible-yarn-testing",
  "version": "1.0.0",
  "license": "MIT",
  "dependencies": {
    "iconv-lite": "^0.4.21",
    "@types/node": "^12.0.0"
  }
}
