{#
Copyright (c) Ansible Project
GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
SPDX-License-Identifier: GPL-3.0-or-later
#}
terraform {
  required_providers {
    {{ tf_provider['name'] }} = {
      source  = "{{ tf_provider['source'] }}"
      version = "{{ tf_provider['version'] }}"
    }
  }
}
